/**
	The class implements a set of methods of the AxisJoystick.h
	interface for working with a joystick controller.

	https://github.com/YuriiSalimov/AxisJoystick

	Created by Yurii Salimov, February, 2018.
	Released into the public domain.
*/
#include "AxisJoystick.h"

AxisJoystick::AxisJoystick(
	const int SW_pin,
	const int VRx_pin,
	const int VRy_pin
) {
	this->SW_pin = SW_pin;
	this->VRx_pin = VRx_pin;
	this->VRy_pin = VRy_pin;
	init();
}

inline void AxisJoystick::init() {
	pinMode(this->SW_pin, INPUT_PULLUP);
	pinMode(this->VRx_pin, INPUT);
	pinMode(this->VRy_pin, INPUT);
}

/**
	Single reading of the joystick controller.
	If the joystick is clamped, the next
	value of pressing - NOT.
	@return value of pressing the joystick:
		Move::PRESS - button is pressed;
		Move::UP - X axis is pressed up;
		Move::DOWN - X axis is pressed down;
		Move::RIGTH - Y axis is pressed right;
		Move::LEFT - Y axis is pressed left;
		Move::NOT - not pressed.
*/
AxisJoystick::Move AxisJoystick::singleRead() {
  const Move joystickMove = multipleRead();
  if (joystickMove != this->previousMove) {
    this->previousMove = joystickMove;
		return joystickMove;
  }
  return Move::NOT;
}

/**
	Multiple reading of the joystick controller.
	@return value of pressing the joystick:
		Move::PRESS - button is pressed;
		Move::UP - X axis is pressed up;
		Move::DOWN - X axis is pressed down;
		Move::RIGTH - Y axis is pressed right;
		Move::LEFT - Y axis is pressed left;
		Move::NOT - not pressed.
*/
AxisJoystick::Move AxisJoystick::multipleRead() {
	if (isPress()) {
		return Move::PRESS;
	} else if (isUp()) {
		return Move::UP;
	} else if (isDown()) {
		return Move::DOWN;
	} else if (isRight()) {
		return Move::RIGHT;
	} else if (isLeft()) {
		return Move::LEFT;
	}
	return Move::NOT;
}

boolean AxisJoystick::isPress() {
	return readSW() == JOYSTICK_BUTTON_PRESS_SIGNAL;
}

boolean AxisJoystick::isUp() {
	return readVRx() >= JOYSTICK_AXIS_HIGH_RANGE;
}

boolean AxisJoystick::isDown() {
	return readVRx() <= JOYSTICK_AXIS_LOW_RANGE;
}

boolean AxisJoystick::isRight() {
	return readVRy() >= JOYSTICK_AXIS_HIGH_RANGE;
}

boolean AxisJoystick::isLeft() {
	return readVRy() <= JOYSTICK_AXIS_LOW_RANGE;
}

int AxisJoystick::readVRx() {
	return analogRead(this->VRx_pin);
}

int AxisJoystick::readVRy() {
	return analogRead(this->VRy_pin);
}

int AxisJoystick::readSW() {
	return digitalRead(this->SW_pin);
}

// @Deprecated
int AxisJoystick::xAxis() {
	return readVRx();
}

// @Deprecated
int AxisJoystick::yAxis() {
	return readVRy();
}
